
%macro collate (data=, var=, suf=, out=out, delete=);

/* Updates Jan 2013 to allow shorthand specification of datasets and suffixes

   Example:
   data = data1-data5 (same as data1 data2 data3 data4 data5);
   suf = 5*4 (same as 4 4 4 4 4);

   Restriction: Only works up to dataset 99, i.e data70-data99 works but
                not data70-data100;


/*    This macro takes 2 or more datasets with common vector name variables
      and concatenates them across into one dataset, with the array lengths
      extended to include the variables from all data sets.  This macro
      can be useful when several %breakdn's have been run, and you wish
      to simplify the %nmove statements in the report section.

      data = list of datasets to concatenate
      var  = list of common prefix names
      suf  = a list of the number of array elements in each dataset

Example:

     Say DATA1 contains variables m1-m10 n1=n10 and
         DATA2 contains variables m1-m5  n1=n5

     %collate(data=data1 data2, var= m n, suf = 10 5, out=alldata);

     Data ALLDATA will contain variables m1-m15, n1-n15 where

     m1-m10 and n1-n10 will be the same as in DATA1 and
     m11-m15 and n11-n15 are the values of m1-m5 and n1-n5 from DATA2

     Use:

     %breakdn(data=temp, class= agecat 4, var = death, out=out1);
     %breakdn(data=temp, class= sex 2, var = death, out=out2);
     %breakdn(data=temp, class= race 4, var = death, out=out3);

      %collate (data=out1 out2 out3, var= m sum, suf=4 2 4, out=out);

      data OUT will contain variables sum1-sum10 and m1-m10 containing
      the number and percent of deaths across the categories of age, sex,
      and race.

*/

%local i j k l y z keeplist;
%local xdata;

%do i = 1 %to 100;
 %let datatemp = %scan(&data,&i,' ');
 %put datatemp = &datatemp;
 %if %length(&datatemp) = 0 %then %goto donedata;
 %if %index(&datatemp,-) > 0 %then %do;
  %let word1 = %scan(&datatemp,1,-);
  %let word1_length = %length(&word1);
  %let y = %sysfunc(anydigit(&word1,-&word1_length));
  %let z = %sysfunc(anydigit(&word1,-&word1_length+1));
  %if %eval(&y-&z) = 1 %then %let word1_root = %sysfunc(substr(&word1,1,&z-1));
  %else %let word1_root = %sysfunc(substr(&word1,1,&y-1));
  %if %eval(&y-&z) = 1 %then %let word1_suffix = %sysfunc(substr(&word1,&z,2));
  %else %let word1_suffix = %sysfunc(substr(&word1,&y,1));

  %let word2 = %scan(&datatemp,2,-);
  %let word2_length = %length(&word2);
  %let y = %sysfunc(anydigit(&word2,-&word2_length));
  %let z = %sysfunc(anydigit(&word2,-&word2_length+1));
  %if %eval(&y-&z) = 1 %then %let word2_root = %sysfunc(substr(&word2,1,&z-1));
  %else %let word2_root = %sysfunc(substr(&word2,1,&y-1));
  %if %eval(&y-&z) = 1 %then %let word2_suffix = %sysfunc(substr(&word2,&z,2));
  %else %let word2_suffix = %sysfunc(substr(&word2,&y,1));

  %put word1=&word1;
  %put word1_length=&word1_length;
  %put word2=&word2;
  %put word2_length=&word2_length;
  %put word1_root= &word1_root;
  %put word2_root= &word2_root;
  %put word1_suffix = &word1_suffix;
  %put word2_suffix = &word2_suffix;

 %do j = &word1_suffix %to &word2_suffix;
  %let xdata = &xdata &word1_root&j;
 %end;
 %end;

 %if %index(&datatemp,-) = 0 %then %do;
  %let word2 = &datatemp;
  %let xdata= &xdata &word2;
 %end;

%end;
 %donedata:

 %let data = &xdata;
 %put data = &data;

 %local xsuf;

%put suf = &suf;
%do i = 1 %to 100;
 %let suftemp = %scan(&suf,&i,' ');
 %put suftemp = &suftemp;
 %if &suftemp = %str() %then %goto done;
 %if %index(&suftemp,*) > 0 %then %do;
  %let word1 = %scan(&suftemp,1,*);
  %let word2 = %scan(&suftemp,2,*);
  %do j = 1 %to &word1;
   %let xsuf = &xsuf &word2;
  %end;
 %end;
 %if %index(&suftemp,*) = 0 %then %do;
 %let word2 = &suftemp;
  %let xsuf = &xsuf &word2;
 %end;
%end;
%done:

%let suf = &xsuf;
%put suf =&suf;

data &out ;
 %ntok(&data,ndata);
 %ntok(&var,nvar);
 %ntok(&suf,nsuf);
 %let keeplist = ;

 %let sufvala = 0;
 %do i = 1 %to &ndata ;
   %let dataset = %scan(&data,&i);
   %let sufval  = %scan(&suf,&i);
    set &dataset ( rename = (
     %do j = 1 %to &nvar;
      %let varname = %scan(&var,&j);
       %do k = 1 %to &sufval;
         %let l = %eval(&sufvala+&k);
         &varname&k = _&varname&l
	 %let keeplist = &keeplist _&varname&l;
       %end;
     %end;
     ) );
   %let sufvala = %eval(&sufvala+&sufval);
   %end;
   keep &keeplist;
 proc datasets;
  modify &out;
  rename
  %do j = 1 %to &nvar;
   %let varname = %scan(&var,&j);
   %do k = 1 %to &sufvala;
    _&varname&k = &varname&k
   %end;
  %end;
;

%let delete = %upcase(&delete);
%if &delete = Y %then %do;
 proc delete data=&data;
%end;

%mend collate;

