/**************************************************************************
    MACRO GENMODP
     GENMODP runs PROC GENMOD and stores statistics from
     the model into one or more SAS datsets. These can then be moved  
     to a report page. Default parameters set for Poisson regression 
     Written March 2009 (GG)
**************************************************************************/
%macro genmodp(data=_last_,
               class=,
               dlist=, ilist=, blist=,                          
	       link=log, dist=poisson, offset=,
	       repeated=,
	       factor=,out=&factor,
	       estimate=, estdata=,
	       units=,
	       where=,
               options=);
                                                                                
/**********************************************************************
Example:                                                                  

%genmodp(data=temp,
        class= ptid yearcat,
        dlist= nhosp naids, 
	ilist= yearcat cd4 , 
	offset = lpr,
	repeated = subject=pin/type=cs corrw, 
	factor= yearcat ,
	out=out1 ,
	estimate = '2005 v 2006' yearcat 1 -1 0|'2006 v 2007' yearcat 0 1 -1,
	estdata=est);

     Parameters to macro :

     data   = data set to be read                                               
     class  = class variable(s)                                         
     dlist  = dependent variable list - a seperate GENMOD is run for             
              each variable in dlist                                            
     ilist  = independent variable list (same for each dep. var)                
     blist  = (optional) if given, the i'th variable is added as                
              a covariate to the i'th analysis. 
     link   = link function (default is log)
     dist   = distribution (default is poisson)
     offset = (optional) offset variable name 
     repeated = full repeated statement (see example) If left
                blank no repeated statement will be generated.
     units  = before the exp(beta) are calculated the beta is
              multiplied by this value. This is given as a list of values
	      over the number of factors.
     estimate = The estimate statement if desired (see example).  
                Multiple estimate statments are separated by a |.
     factor = independent variable(s) for which statistics are to be            
              stored. This is stored for each analysis performed.               
     out    = output SAS dataset(s) in which the statistics are stored.         
              There is one dataset for each factor with one                     
              observation in each dataset.                                      
     If the factor is a class variables the output dataset contains

     p1-p(n) pvalues for that factor across dependent variables
     k1-k(n) Chi-Square statistics for that factor

     If the factor is NOT a class variables the output datset contains

     e1-e(n)   regression coefficients
     se1-se(n) standard errors
     z1-z(n)   z-statistics
     p1-p(n)   p-values
     r1-r(n)   exp(coef)
     l1-l(n)   lower 95% limit for exp(coef)
     u1-u(n)   upper 95% limit for exp(coef)

     n=the number of analyses run (= number of dep var's).             


     estdata = the SAS datset containing the estimate statistics. The
               variables on this dataset are:
	       
     e1-e(n)   estimate of the contrast
     se1-se(n) standard errors of the contrast
     z1-z(n)   z-statistic of the contrast
     p1-p(n)   p-value for the contrast
     r1-r(n)   exp(estimate)
     l1-l(n)   lower 95% limit for exp(estimate)
     u1-u(n)   upper 95% limit for exp(estimate)

     n is the total number of estimate statements times the number of
     dependent variables. They are stored in the order generated. If 
     there are 2 dependent variables and 2 contrasts then  n=4 and
     
     e1 = estimate for analyses 1, contrast 1
     e2 = estimate for analyses 1, contrast 1
     e3 = estimate for analyses 2, contrast 1
     e4 = estimate for analyses 2, contrast 2

	
     where   = where clause to subset observations                              
               (set where to "where sex = 1" to do analyses on men)
     options  = options added to the PROC GENMOD statement.

                                                                                
**********************************************************************/

*ods output GEEEmpPEst (match_all=_sol_ persist=proc) = _sol_;
ods output GEEEmpPEst (match_all=_sol_ persist=proc) = _sol_;
ods output Type3      (match_all=_tests_ persist=proc) = _tests_ ;
ods output Estimates (match_all=_estimates_ persist=proc)= _estimates_ ;
*ods output Contrasts (match_all=_contrasts_ persist=proc)= _contrasts_ ;

data _null_;                                                                    

%local  _estimates_ _sol_ _tests_ i j k dep bas ;
%let dlist = %upcase(&dlist) ;                                                  
%let factor = %upcase(&factor) ;                                                
%ntok(&factor,nfactor,vname=factor);
%ntok(&out   ,nout   ,vname=out);                                               
%ntok(&dlist ,ndep);                                                            
%ntok(&blist ,nbas);
%ntok(&estimate ,nest,vname=estimate,token=%str(|));
                                               
%if &class eq %str() or &dlist eq %str() or %length(&ilist) = 0                 
    or %length(&factor)  eq 0  or &out   eq %str() %then %do ;                  
 ERROR '*** SOME PARAMETERS NOT SPECIFIED - SEE MACRO CALL ***';                
 ABORT;                                                                         
 ENDSAS;                                                                        
%end;                                                                           
                                                                                
%if &blist ne %str() %then %do;                                                 
%if &ndep ne &nbas %then %do;                                                   
 ERROR '*** NUMBER OF DEPENDENT AND BASELINE VARIABLES NOT = ***';              
 ABORT;                                                                         
 ENDSAS;                                                                        
%end;                                                                           
%end;                                                                           
                                                                                
%if &nout   ne &nfactor %then %do ;                                             
 ERROR '*** NUMBERS OF FACTORS AND OUTPUT DATASETS NOT =  ***';                 
 ABORT;                                                                         
 ENDSAS;                                                                        
%end;                                                                           

%if &units = %str() %then %do;                                                  
 %do i = 1 %to &nfactor ;                                                       
  %let units&i = 1 ;                                                            
 %end;                                                                          
%let nunits=&nfactor;                                                           
%end;                                                                           
%else %do;                                                                      
 %ntok(&units,nunits,vname=units);                                              
%end;
 
 %do i = 1 %to &ndep;                                                           
  %let dep = %scan(&dlist,&i);                                                  
  %let bas = %scan(&blist,&i);                                                  
  proc genmod data=&data &options ;
   class &class;                                                                
   model &dep = &ilist &bas  /link=&link dist=&dist type3 wald
   %if %length(&offset) > 0 %then %do;
   offset=&offset                                               
   %end;
   ;

   %if %length(&repeated) > 0 %then %do;
   repeated &repeated ;
   %end;
   
   %if %eval(&nest) gt 0 %then %do;
   %do j = 1 %to &nest; 
    estimate &&estimate&j ;
   %end;
   %end;

   %if %length(&where) ne 0 %then %do;
    &where;
   %end;

 %end;
  run;
 data _sol_ ;
  set &_sol_;
 data _tests_ ;
  set &_tests_;
 run;

 %if %eval(&nest) gt 0 %then %do;
 data _estimates_;
  set &_estimates_;
  if substr(Label,1,3) = 'Exp' then delete;
 run;
 proc contents;
 data &estdata ;
 %let upp = %eval(&nest*&ndep);
  keep e1-e&upp se1-se&upp z1-z&upp p1-p&upp r1-r&upp u1-u&upp L1-L&upp ;
  array e(*) e1-e&upp;
  array se(*) se1-se&upp;
  array z(*) z1-z&upp;
  array p(*) p1-p&upp;
  array r(*) r1-r&upp;                                                        
  array u(*) u1-u&upp;                                                        
  array l(*) l1-l&upp;                                                        
  z05 =PROBIT(1-0.05/2);
  do i = 1 to totobs;
   set _estimates_ point=i nobs=totobs;
    e(i) = MeanEstimate ;
    se(i) = StdErr ;
    z(i) = sqrt(ChiSq) ;
    if MeanEstimate < 1 then z(i) = -1*z(i);
    p(i) = ProbChiSq;
    r(i) = MeanEstimate;
    u(i) = MeanUpperCL;                                   
    l(i) = MeanLowerCL;
  end;
  output;
  stop;
  run;
 %end;


%do j = 1 %to &nfactor ;                                                        
 data &&out&j;                                                                  
  %if %index(%upcase(&class), &&factor&j) gt 0 or
      %index(&&factor&j,*) gt 0 %then %do;
  set _tests_ ;
  if upcase(source) = "&&factor&j" ;
 data &&out&j;
  keep source p1-p&ndep k1-k&ndep ;
  array p(*) p1-p&ndep;                                                         
  array ka(*) k1-k&ndep;
  do i = 1 to &ndep ;                                                           
   set &&out&j point=i ;                                                        
     p(i) = ProbChisq ;
     ka(i) = ChiSq ;
  end;                                                                          
  output;                                                                       
  stop;                                                                         
  %end;
  %else %do;
  set _sol_;
  if upcase(parm) = "&&factor&j" ;
 data &&out&j;                                                                  
  keep parm p1-p&ndep e1-e&ndep se1-se&ndep z1-z&ndep r1-r&ndep u1-u&ndep L1-L&ndep;
  array e(*) e1-e&ndep;                                                         
  array p(*) p1-p&ndep;                                                         
  array se(*) se1-se&ndep;  
  array zv(*) z1-z&ndep;                                                        
  array r(*) r1-r&ndep;                                                        
  array u(*) u1-u&ndep;                                                        
  array l(*) l1-l&ndep;                                                        
  z05 =PROBIT(1-0.05/2);
  do i = 1 to &ndep ;                                                           
   set &&out&j point=i ;                                                        
     e(i) = Estimate ;
     p(i) = ProbZ ;
     se(i) = Stderr;
     zv(i) = Z;
     r(i) = exp(&&units&j * e(i));
     u(i) = exp( &&units&j* (e(I)+z05*se(I)) );                                    
     l(i) = exp( &&units&j* (e(I)-z05*se(I)) );
  end;                                                                          
  output;                                                                       
  stop;                                                                         
  %end;
  run;                                                                          
%end;                                                                           
/*
proc datasets;
 delete  _sol_ _tests_ 
   %if %eval(&nest) gt 0 %then %do;
   _estimates_
  %end;
  ;
run;
*/
ods output close;
%mend ;                                                                         
