

/***************    MACRO LIFETAB ****************************************

     %macro lifetab  (data=, strata = , t=, times= , inter =,
                      ninter = , out = &data, print=Y, zero=);

     Macro LIFETAB reads the dataset output in the OUTSURV option of
     PROC LIFETEST and outputs the survival values for specified time
     values for each strata. The user need to run PROC LIFETEST before
     calling the macro:

  The output dataset contains has one observation and variables:

     s1-sk = survival values where k is the product of the number
             of strata (STRATA) and the number of time points
             evaluated.
     f1-fk = 1 - s
     t1-tk = times evaluated at (these repeat for each value of STRATA)
     g1-gk = strata value
     u1-uk = (1-alpha)% upper confidence limit for s
     l1-lk = (1-alpha)% lower confidence limit for s

      Parameters:

      DATA     = data set containing points of survival table
      STRATA   = strata or group variable
      T        = name of variable containing time of outcome
      TIMES    = list of times to output survival curve for each strata
      INTER    = output survival values at NINTER intervals of this length
      NINTER   = number of INTERVALs
      ZERO     = Y outputs S and F at T=0
      OUT      = output dataset containing s/f/t/g
      PRINT    = Y prints out values of survival curve at all uncensored
                   points and sepatately for points requested

      Use TIMES or INTER with NINTER to indicate the times you want
      the survival curve to be outputed. The units used should be consistant
      with the units used for T (i.e. days, months, years, etc.).

      Example: TIMES = .5 1 2 3 4 5 (output at 6 months and then yearly)

      Example: NINTER = 5
               INTER = 365   (output at 365,365*2, 365*3, 365*4, 365*5

*********************************************************************
Example :

proc lifetest notable outsurv=surv  ; time t*chd(0) ; strata group;

%lifetab (data=surv, inter=365, ninter=7 ,  out=out, strata=group);
****************************************************************************/


 %macro lifetab (data=, strata = , t=t, times= , inter =, ninter=,
                   out = &data, print=Y, zero=);

 %if %length(&times) = 0 %then %do;
  %let times = %str();
  %if %upcase(&zero) = Y %then %let start=0; %else %let start= 1;
  %do i = &start %to &ninter;
    %let times = &times %eval(&inter * &i);
  %end;
 %end;

 %ntok(&times,numinter,vname=times);

 data &out;
   set &data;
   if _censor_ eq 1 then delete;
   f=1-survival;
   s=survival;
   ucl = sdf_ucl ;
   lcl = sdf_lcl ;
   keep f s &strata &t ucl lcl ;
%if &print = Y %then %do;
 proc print;
%end;
run;
data &out;
 ninter = &numinter;
 array times(*) times1 - times&numinter (&times) ;
 retain oldf cutoff i ;
 keep oldf cutoff &strata olds olducl oldlcl ;
 oldf = f;
 olds = 1 - oldf;
 olducl = ucl ;
 oldlcl = lcl ;
 set &out ; by &strata;
 if first.&strata then do;
  oldf = 0 ; i = 1; cutoff= &times1 ;
 end;

 do while ( &t gt cutoff and i le ninter );
  output;
  i = i + 1;
  if i gt ninter then return;
  cutoff = times(i);
 end;

 if last.&strata and &t le cutoff then do;
  do while (i le ninter);
   oldf = f;
   olds = 1-oldf;
   olducl = ucl ;
   oldlcl = lcl ;
   output ;
   i = i + 1;
   if i gt ninter then return;
   cutoff = times(i) ;
  end;
 end;
%if &print = Y %then %do;
 proc print;
%end;

proc transpose data=&out prefix=f out=temp_1; var oldf;
proc transpose data=&out prefix=t out=temp_2; var cutoff;
proc transpose data=&out prefix=g out=temp_3; var &strata;
proc transpose data=&out prefix=s out=temp_4; var olds;
proc transpose data=&out prefix=u out=temp_5; var olducl ;
proc transpose data=&out prefix=l out=temp_6; var oldlcl ;
data &out;
 set temp_1;
 set temp_2;
 set temp_3;
 set temp_4;
 set temp_5;
 set temp_6;
 drop _name_;
run;
proc delete data=temp_1 temp_2 temp_3 temp_4 temp_5 temp_6 ;
run;

%mend lifetab;


