/**************************************************************************

   Update Aug 9, 2016 to add estimates and contrasts

   Update July 9, 2010 to add namelen=32 to account for possible
   long names.
   Updates June 12, 2006 to work for both version 8 and version 9
   Separate code is run depending on version - macro variable sysver 

   Updated June 2005 to add counts of number of events, non-events and
   total to each dataset
   
   Updated March 2001 to simplify code using ODS.
   Also added class statement and Likelihood    
   Ratio Chi-Square and P-value for testing all covariates.
   Works in Version 8 only.
**************************************************************************/

%macro logistp(data=_last_,dlist=,ilist=, units=, class=,                       
               factor=, out=&factor,where=, order=,
	       estimate=,estdata=,
	       contrast=,condata=);
/*
Example:

%logistp(data=temp, dlist=fever headache, class=group,
         ilist=age sex group, factor=age group,order=d);


   Overview:
   LOGISTP runs proc LOGIST(s) to perform logistic regression and saves
   results for factors of interest into SAS datasets.

   Description of Parameters:
 
     data   = SAS data set to be read
     dlist  = dependent variable list. Each variable should have 2 values,
              a value for the event and a value for a non-event.
     order  = if set to D then the event indicator is the largest value in the
              variables in dlist. 
              Eg. fever = 0 1 with D option computes probability of a value of 
	      1 for fever.  The default is the smallest value.
     ilist  = independent variable list. This list will be used for each
              dependent variable given in dlist.
     factor = independent variable(s) for which statistics are to be
              stored.
     units  = before the odds ratios are calculated the beta is
              multiplied by this value. This is given as a list of values
	      over the number of factors.
     where   = where clause to subset observations
               (set where to "where sex = 1" to do analyses on men)
     out    = SAS dataset(s) in which statistics are stored for each variable
              listed in factor.  There is one dataset for each factor with one
              observation in each dataset. 

	      If the factor is a class variable then the variables are:
	      (where n is number of analyses - dependent variables)

	      CHISQ1-CHISQn = Wald Chi-Sqare for the factor (n analyses)
	      P1-Pn         = P-values associated with Chi-Square
	      LRCHI1-LRCHIn = Likelihood Ratio Chi-square (put on all
	                      datasets) for global test
              PLR1-PLRn     = P-values for LR statistic	      
	      NE1-NEn       = Number of events (Value modeled as Prob) in analysis
	      NC1-NCn       = Number of non-events in analysis
	      NT1-NTn       = Total Observations in analyses

              If the factor is not a class variable then the variables are:

	      E1-En   = The regression coefficients (n analyses)
	      SE1-SEn = Standard errors of coefficients
	      Z1-Zn   = Z-statistics (e/se);
	      P1-Pn   = P-values corresponding to Z
	      O1-On   = Odds Ratios (exp(unit*e))
	      U1-Un   = 95% Upper CI for Odds Ratio
	      L1-Ln   = 95% Lower CI for Odds ratio
	      LRCHI1-LRCHIn = Likelihood Ratio Chi-square (put on all
	                      datasets) for global test
              PLR1-PLrn = P-values for LR statistic	      
	      NE1-NEn       = Number of events (Value modeled as Prob) in analysis
	      NC1-NCn       = Number of non-events in analysis
	      NT1-NTn       = Total Observations in analyses


      estimate = The estimate statement if desired. Multiple estimate statements
                 are separated by a /

                 example: estimate = 'A vs B' group 1 0 -1/'A vs C' group 0 1 -1
       
       estdata = the SAS dataset containing the estimate statistics, The variables
                 on this dataset are:

		 e1-en = estimate value
		 se1-sen = standard error of the estimate
		 z1-zn = estimate/SE
		 p1-pn = pvalues
		 o1-on = odds ratio (exp(e))
		 l1=ln = lowee 95% CI
		 u1-un = upper 95% CI
		 
	       n is the number of estimates statements times the number of dependent
	       variables. They are stored in the order generated. If there are 2 dependent
	       variables and 2 estimates then n=4 and

               e1 = estimate for analyses 1, contrast 1
               e2 = estimate for analyses 1, contrast 2
               e3 = estimate for analyses 2, contrast 1
               e4 = estimate for analyses 2, contrast 2

     contrast = Contrast statement if desired. These can be 1 or >1 DF contrasts.
                Mutliple contrasts are separated by a /. If the contrast is 1 df
		then the syntax is identical to the estimate statement. If the
		contrast is > 1 df then the form is:

		contrast = %str('2 df test' group 1 -1 0,group 0 1 -1);

     condata = the SAS dataset containing the contrast statistics. The variables on
               the dataset are:

	       chisq1-chisqN= Chi-square for the contrast
	       p1-pN = p values for the contrasts
	       
	       n is the number of contrast statements times the number of dependent
	       variables. They are stored in the same order as in the estimate
	       statements.


*/

run;
ods output ParameterEstimates (persist=proc)= _est_ ;
ods output GlobalTests (persist=proc)= _gtests_ ;
ods output Type3 (persist=proc) = _TypeIII_ ;
ods output ResponseProfile (persist=proc) = _ResponseProfile_ ;
ods output Estimates (persist=proc) = _estimates_;
ods output ContrastTest (persist=proc) = _contrasts_;
                                                                                
data _null_;                                                                    
%local i j _typeiii_ ;
%let dlist = %upcase(&dlist) ;                                                  
%let factor = %upcase(&factor) ;                                                
%let class = %upcase(&class) ;                                                
%let order  = %upcase(&order) ;

%let descend = %str();
%if &order = D or &order = DESCENDING %then %let descend = descending;
                                                                                
%ntok(&factor,nfactor,vname=factor);                                            
%ntok(&out   ,nout   ,vname=out);                                               
%ntok(&dlist ,ndep);                                                            
%ntok(&estimate, nest, vname=estimate,token=%str(/));
%ntok(&contrast, ncon, vname=contrast,token=%str(/));
                                                                                
%if &dlist eq %str() or %length(&ilist) = 0                                     
    or %length(&factor)  eq 0  or &out eq %str() %then %do ;                    
 ERROR '*** SOME PARAMETERS NOT SPECIFIED - SEE MACRO CALL ***';                
 ABORT;                                                                         
 *ENDSAS;                                                                        
%end;                                                                           
                                                                                
%if &nout   ne &nfactor %then %do ;                                             
 ERROR '*** NUMBERS OF FACTORS AND OUTPUT DATASETS NOT =  ***';                 
 ABORT;                                                                         
 *ENDSAS;                                                                        
%end;                                                                           
                                                                                
%if &units = %str() %then %do;                                                  
 %do i = 1 %to &nfactor ;                                                       
  %let units&i = 1 ;                                                            
 %end;                                                                          
%let nunits=&nfactor;                                                           
%end;                                                                           
%else %do;                                                                      
 %ntok(&units,nunits,vname=units);                                              
%end;                                                                           
                                                                                
 %do i = 1 %to &ndep;                                                           
  %let dep = %scan(&dlist,&i);                                                  
  proc logistic  &descend data=&data namelen=32;
   %if &class ne %str() %then %do;
     class &class/param=glm;
   %end;
   model &dep = &ilist ;                                                        
   %if %eval(&nest) gt 0 %then %do;
   %do j = 1 %to &nest;
    estimate &&estimate&j/exp cl;
   %end;
   %end;
   %if %eval(&ncon) gt 0 %then %do;
   %do j = 1 %to &ncon;
    contrast &&contrast&j;
   %end;
   %end;
   &where;                                                                      
   run;
 %end;                                                                          
 ods output close;

 data _nevents_ _ncensored_ ;
  set _ResponseProfile_;
  if OrderedValue = 1 then output _nevents_;
  if OrderedValue = 2 then output _ncensored_;
 run;
 data _gtests_;
  set _gtests_;
  if upcase(Test) = 'LIKELIHOOD RATIO';
  keep ChiSq ProbChiSq;;
 run;

%do j = 1 %to &nfactor ;                                                        
 data &&out&j;
 %if %index(%upcase(&class), &&factor&j) gt 0 or
     %index(&&factor&j,*) gt 0 %then %do;
  set _TypeIII_ ;
  *if upcase(variable) = "&&factor&j";
  if upcase(effect) = "&&factor&j";
 data &&out&j;
  keep effect chisq1-chisq&ndep p1-p&ndep
       lrchi1-lrchi&ndep plr1-plr&ndep 
       ne1-ne&ndep nc1-nc&ndep nt1-nt&ndep;
  array p(*) p1-p&ndep;
  array c(*) chisq1-chisq&ndep;
  array lrchi(*) lrchi1-lrchi&ndep ;
  array plr(*) plr1-plr&ndep;     
  array ne(*) ne1-ne&ndep;
  array nc(*) nc1-nc&ndep;
  array nt(*) nt1-nt&ndep;
  do i = 1 to &ndep;
   set &&out&j point=i;
    p(i) = ProbChiSq;
    c(i) = WaldChiSq;
   set _gtests_ point=i;
    lrchi(i) = ChiSq;
    plr(i) = ProbChisq;
   set _nevents_ point=i;
    ne(i) = Count;
   set _ncensored_ point=i;
    nc(i) = Count;
    nt(i) = ne(i) + nc(i);
  end;
  output;
  stop;
 %end;

 %else %do;  
  set _est_;                                                                    
  if upcase(Variable) = "&&factor&j";
 data &&out&j;                                                                  
  keep Variable p1-p&ndep e1-e&ndep se1-se&ndep z1-z&ndep                      
       l1-l&ndep u1-u&ndep o1-o&ndep                                           
       lrchi1-lrchi&ndep plr1-plr&ndep
       ne1-ne&ndep nc1-nc&ndep nt1-nt&ndep;
  array e(*) e1-e&ndep;                                                         
  array p(*) p1-p&ndep;                                                         
  array s(*) se1-se&ndep;                                                       
  array z(*) z1-z&ndep;                                                         
  array l(*) l1-l&ndep;                                                         
  array u(*) u1-u&ndep;                                                         
  array o(*) o1-o&ndep;                                                         
  array lrchi(*) lrchi1-lrchi&ndep ;
  array plr(*) plr1-plr&ndep;     
  array ne(*) ne1-ne&ndep;
  array nc(*) nc1-nc&ndep;
  array nt(*) nt1-nt&ndep;
                        
  z05 = probit(1-0.05/2);
  do i = 1 to &ndep;                                                            
   set &&out&j point=i ;                                                        
   set _gtests_ point=i;
   set _nevents_ point=i;
    ne(i) = Count;
   set _ncensored_ point=i;
    nc(i) = Count;
    nt(i) = ne(i) + nc(i);
   e(i) = Estimate ;                                                          
   s(i) = StdErr;                                                     
   z(i) = e(i)/s(i);                                                            
   p(i) = 1-probnorm(abs(z(i)));                                                
   p(i) = 2*p(i);                                                               
   lrchi(i) = ChiSq;
   plr(i) = ProbChisq;
   o(i) = exp( &&units&j* e(i));                                                
   u(i) = exp( &&units&j* (e(I)+z05*s(I)) );                                    
   l(i) = exp( &&units&j* (e(I)-z05*s(I)) );                                    
  end;                                                                          
  output;                                                                       
  stop;                                                                         
  %end;
  run;                                                                          
%end;                                                                           

%if %eval(&nest) gt 0 %then %do;
 data &estdata;
 %let upp = %eval(&nest*&ndep);
 keep e1-e&upp s1-s&upp z1-z&upp p1-p&upp
      o1-o&upp u1-u&upp l1-l&upp;
 array e(*) e1-e&upp;
 array s(*) s1-s&upp;
 array z(*) z1-z&upp;
 array p(*) p1-p&upp;
 array o(*) o1-o&upp;
 array u(*) u1-u&upp;
 array l(*) l1-l&upp;
 do i = 1 to totobs;
  set _estimates_ point=i nobs=totobs;
  e(i) = Estimate;
  s(i) = StdErr;
  z(i) = zValue;
  p(i) = Probz;
  o(i) = ExpEstimate;
  u(i) = UpperExp;
  l(i) = LowerExp;
 end;
 output;
 stop;
 run;
%end;

%if %eval(&ncon) gt 0 %then %do;
 data &condata;
 %let upp = %eval(&ncon*&ndep);
 keep chisq1-chisq&upp p1-p&upp;
 array k(*) chisq1-chisq&upp;
 array p(*) p1-p&upp;
 do i = 1 to totobs;
  set _contrasts_ point=i nobs=totobs;
  k(i) = WaldChiSq;
  p(i) = ProbChiSq;
 end;
 output;
 stop;
 run;
%end;

proc delete data = _est_ _gtests_ _nevents_ _ncensored_;
run;
%if %sysfunc(exist(work._TypeIII_)) %then %do;
 proc delete data=  _typeIII_;
 run;
%end;

ods output close;
%mend ;                                                                         

