
%macro medianp(data=_LAST_ , out=OUT, class=, var=, test=W,
               print=NOPRINT, where=);
/* 
   This macro puts the P-value and degree of freedom generated by
   PROC NPAR1WAY into a SAS data set. 
   Updated 22Aug02 to add Chisq to dataset
   
   data   = data set to read, the default is to use the last dataset
   out    = output data set with 
            P-values (p1-p?)
	    degree of freedom (df1-df?)
	    Chi-square Value (chisq1-chisq?)
	    for each dependable variables listed in the VAR.
	    The default output dataset is OUT
   class  = name ONE and only one classification variable (EX. class=sex)
   var    = a list of the response or dependent variables to be analyzed
   test   = four choices -- M(Median); S(Savage); V(VW); W(Wilcoxon)
            the default analyses choice is W (Wilcoxon)
   print  = the default is NOPRINT, but you can change it to Y or y to look
            at the PROC NPAR1WAY result
   where  = where clause to subset observations
            e.g. where= where sex = 1

   Sample call: %medianp(data=first, out=pout, class=randgrp, var=cd4bl rnabl,
                         where= where unit eq 10);
*/

/* allow upper or lower cases for the TEST input */
%let test=%upcase(&test);

/* allow NOPRINT as an option */
%if %upcase(&print) ne NOPRINT %then %let print = %str();

/* get the P and DF from PROC NPAR1WAY */
proc npar1way data=&data &print &test ;
     output  out=_temp_;
     class &class;
     var &var;
     &where;

%If &test = S %then %do;
    %let pvar=P_CHSAV; %let dfvar=DF_CHSAV; %let chisq=_chsav_;
%end;
%if &test = M %then %do;
    %let pvar=P_CHMED; %let dfvar=DF_CHMED; %let chisq=_chmed_ ;
%end;
%if &test= V %then %do;
    %let pvar=P_CHVW; %let dfvar=DF_CHVW; %let chisq = _chvw_ ;
%end;
%if &test= W %then %do;
    %let pvar=P_KW; %let dfvar=DF_KW; %let chisq=_kw_ ;
%end;

/* get the P value out and give it a name */
proc transpose data=_temp_ out=pout prefix=p;
     var &pvar;

/* get the DF value out and give it a name */   
proc transpose data=_temp_ out=dfout prefix=df;
     var &dfvar;

/* get the Chi-square value out and give it a name */   
proc transpose data=_temp_ out=chisqout prefix=chisq;
     var &chisq;
 
/* put P-value and DF into the output dataset &out */
data &out;
     set pout;
     set dfout;
     set chisqout;

proc delete data=_temp_ pout dfout chisqout;

%mend medianp;
