/**************************************************************************
   Updated Dec  2012 to include class statement and estimate and contrast
   statements.
   Updated July 10, 2010 to add namelen=32 option for possible long names
   Updated Oct 09 to accomadate names changes in output variables
   between version 9.1 and 9.2. One line of code is dependent on
   the version variables &sysver
   Updated June 05 to add counts of number of events, non-events 
   (censored observations), and total obs to each dataset

   Updated March 2001 to simplify code using ODS.  Also added Likelihood    
   Ratio Chi-Square and P-value for testing all covariates.

   Feb 2005 - Added run statement in beginning to separate previous non
              macro calls to phreg
**************************************************************************/

   
%macro phregp(data=_last_,dlist=,ilist=, tlist=, units=,strata=, class=,               
              factor=, out=&factor,where=,skipout=,ties=breslow,
	      estimate=,estdata=,contrast=,condata=);

/* Overview:
   PHREGP runs proc PHREG(s) to perform Cox Model regression and saves
   results for factors of interest into SAS datasets.

   Description of Parameters:
 
     data   = SAS data set to be read
     dlist  = dependent variable list. Each variables coded 1 for event 
              and 0 if censored.
     ilist  = independent variable list. This list will be used for each
              dependent variable given in dlist.
     tlist  = time variable list corresponding to the events in dlist. May
              in some cases be the same variable repeated.
     factor = independent variable(s) for which statistics are to be
              stored.
     units  = before the relative risks are calculated the beta is
              multiplied by this value. This is given as a list of values
	      over the number of factors.
     strata = Strata variables used for each regression (eg. clinic).
     where   = where clause to subset observations
               (set where to "where sex = 1" to do analyses on men)
     out    = SAS dataset(s) in which statistics are stored for each variable
              listed in factor.  There is one dataset for each factor with one
              observation in each dataset. 

    If the factor is class variable the output dataset contains:
              P1-Pn = the pvalues (K-1 DF) for that factor across the dep var
	      Chisq1-Chisqp = Wald Chi-square test (K-1 DF)

    If the factor is NOT a class variable the output dataset contains:
	      E1-En = The regression coefficients (n analyses)
	      SE1-SEn = Standard errors of coefficients
	      Z1-Zn   = Z-statistics (e/se);
	      P1-Pn =   P-values corresponding to Z
	      R1-Rn =   Relative Risks (exp(unit*e))
	      U1-Un =   95% Upper CI for RR
	      L1-Ln =   95% Lower CI for RR
	      LRCHI1-LRCHIn = Likelihood Ratio Chi-square (put on all
	                      datasets) for global test
              PLR1-PLrn = P-values for LR statistic	      
	      NE1-NEn       = Number of events in analysis
	      NC1-NCn       = Number of non-events in analysis
	      NT1-NTn       = Total Observations in analyses

      estimate = The estimate statement if desired. Multiple estimate statements
                 are separated by a /

                 example: estimate = 'A vs B' group 1 0 -1/'A vs C' group 1 0 -1
       
       estdata = the SAS dataset containing the estimate statistics, The variables
                 on this dataset are:

		 e1-en = estimate value
		 se1-sen = standard error of the estimate
		 z1-zn = estimate/SE
		 p1-pn = pvalues
		 r1-rn = hazard ratio (exp(e))
		 l1=ln = lowee 95% CI
		 u1-un = upper 95% CI
		 
	       n is the number of estimates statements times the number of dependent
	       variables. They are stored in the order generated. If there are 2 dependent
	       variables and 2 estimates then n=4 and

               e1 = estimate for analyses 1, contrast 1
               e2 = estimate for analyses 1, contrast 2
               e3 = estimate for analyses 2, contrast 1
               e4 = estimate for analyses 2, contrast 2

     contrast = Contrast statement if desired. These can be 1 or >1 DF contrasts.
                Mutliple contrasts are separated by a /. If the contrast is 1 df
		then the syntax is identical to the estimate statement. If the
		contrast is > 1 df then the form is:

		contrast = %str('2 df test' group 1 -1 0,group 0 1 -1);

     condata = the SAS dataset containing the contrast statistics. The variables on
               the dataset are:

	       chisq1-chisqN= Chi-square for the contrast
	       p1-pN = p values for the contrasts
	       
	       n is the number of contrast statements times the number of dependent
	       variables. They are stored in the same order as in the estimate
	       statements.
		
		 
      ties  = method for handling ties. Default is Breslow
      skipout= no datasets are created. Macro used only to write code

Example:

%phregp(data=rand, dlist=chd cvd allcause, ilist = age dbp chol,
        tlist = t t t, strata=clinic, factor=dbp, out=dbp);
%phregp(data=rand, dlist=death nfmi, ilist = trt
        tlist = tdeath tmi, strata=clinic, factor=trt, out=trt);

*/	      
	      
run;
ods output ParameterEstimates (persist=proc) = _est_ ;
ods output GlobalTests (persist=proc) = _gtests_ ;
ods output Type3 (persist=proc)  = _TypeIII_ ;
ods output CensoredSummary (persist=proc)  = _counts_;
ods output Estimates (persist=proc) = _estimates_;
ods output ContrastTest (persist=proc) = _contrasts_;
                                                                                
data _null_;                                                                    

%local i j time;
%let dlist = %upcase(&dlist) ;                                                  
%let factor = %upcase(&factor) ;                                                
%let skipout = %upcase(&skipout);                                               
                                                                                
%ntok(&factor,nfactor,vname=factor);                   
                                                                                
%ntok(&out   ,nout   ,vname=out);                                               
%ntok(&dlist ,ndep);                                          
%ntok(&estimate ,nest,vname=estimate,token=%str(/));
%ntok(&contrast ,ncon,vname=contrast,token=%str(/));

%if &skipout ne Y %then %do ;                                                   
%if &dlist eq %str() or %length(&ilist) = 0                                     
    or %length(&factor)  eq 0  or &out eq %str() %then %do ;                    
 ERROR '*** SOME PARAMETERS NOT SPECIFIED - SEE MACRO CALL ***';                
 ABORT;                                                                         
 ENDSAS;                                                                        
%end;                                                                           
%if &nout   ne &nfactor %then %do ;                                             
 ERROR '*** NUMBERS OF FACTORS AND OUTPUT DATASETS NOT =  ***';                 
 ABORT;                                                                         
 ENDSAS;                                                                        
%end;                                                                           
%end;                                                                           

%if &units = %str() %then %do;                                                  
 %do i = 1 %to &nfactor ;                                                       
  %let units&i = 1 ;                                                            
 %end;                                                                          
%let nunits=&nfactor;                                                           
%end;                                                                           
%else %do;                                                                      
 %ntok(&units,nunits,vname=units);                                              
%end;                                                                           

 %do i = 1 %to &ndep;                                                           
  %let dep = %scan(&dlist,&i);                                                  
  %let time = %scan(&tlist,&i);                                                 
  %if &sysver >= 9.2  %then %do;
  proc phreg data=&data namelen=32;                                  
  %end;
  %else %do;
  proc phreg data=&data ;                                  
  %end;
   %if &class ne %str() %then %do;
    class &class/param=glm;
   %end;
   model &time*&dep(0) = &ilist /ties=&ties rl;
   %if &strata ne %str() %then %do;                                             
    strata &strata;                                                             
   %end;                                                                        
   %if %eval(&nest) gt 0 %then %do;
   %do j = 1 %to &nest;
   estimate &&estimate&j;
   %end;
   %end;
   %if %eval(&ncon) gt 0 %then %do;
   %do j = 1 %to &ncon;
   contrast &&contrast&j;
   %end;
   %end;
   &where;                                                                      
   run;
 %end;                                                                          
 ods output close;

 %if &skipout = Y %then %goto skipout ;                                         

 data _counts_;
  retain control_var '-';
  set _counts_;
  if control_var = '-'  then output;
  keep Total Event Censored;
 run;
 data _gtests_;
  set _gtests_;
  if upcase(Test) = 'LIKELIHOOD RATIO';
  keep ChiSq ProbChiSq;;
 run;
 %if &class ne %str() %then %do;
  data _TypeIII_;
   set _TypeIII_;
  run;
 %end;
 data _est_;
  set _est_;
 run;

%do j = 1 %to &nfactor ;                                                        
 data &&out&j;
 %if %index(%upcase(&class),&&factor&j) gt 0 or
     %index(&&factor&j,*) gt 0 %then %do;
 set _TypeIII_;
 if upcase(effect) = "&&factor&j";
 data &&out&j;
 keep effect chisq1-chisq&ndep p1-p&ndep lrchi1-lrchi&ndep
       plr1-plr&ndep ne1-ne&ndep nc1-nc&ndep nt1-nt&ndep;
 array p(*) p1-p&ndep;
 array c(*) chisq1-chisq&ndep;
 array lrchi(*) lrchi1-lrchi&ndep;
 array plr(*) plr1-plr&ndep;
 array ne(*) ne1-ne&ndep;
 array nc(*) nc1-nc&ndep;
 array nt(*) nt1-nt&ndep;
 do i = 1 to &ndep;
  set &&out&j point = i;
   p(i) = ProbChiSq;
   c(i) = WaldChiSq;
  set _gtests_ point=i;
   lrchi(i) = Chisq;
   plr(i) = ProbChisq;
  set _counts_;
   ne(i) = Event;
   nc(i) = Censored;
   nt(i) = Total;
  end;
  output;
  stop;
 %end;

 %else %do;
  set _est_ ;
 %if &sysver >= 9.2  %then %do;
  if upcase(Parameter) = "&&factor&j" ;
 %end;
 %else %do;
  if upcase(Variable) = "&&factor&j" ;
 %end;
 data &&out&j;
  keep p1-p&ndep e1-e&ndep se1-se&ndep z1-z&ndep                                
       l1-l&ndep u1-u&ndep r1-r&ndep lrchi1-lrchi&ndep 
       plr1-plr&ndep
       ne1-ne&ndep nc1-nc&ndep nt1-nt&ndep;
  array e(*) e1-e&ndep;                                                         
  array p(*) p1-p&ndep;                                                         
  array s(*) se1-se&ndep;                                                       
  array z(*) z1-z&ndep;                                                         
  array l(*) l1-l&ndep;                                                         
  array u(*) u1-u&ndep;                                                         
  array r(*) r1-r&ndep;                                                         
  array lrchi(*) lrchi1-lrchi&ndep ;
  array plr(*) plr1-plr&ndep;     
  array ne(*) ne1-ne&ndep;
  array nc(*) nc1-nc&ndep;
  array nt(*) nt1-nt&ndep;

  z05 =PROBIT(1-0.05/2);                                                        
  do i = 1 to &ndep;                                                            
   set &&out&j point=i ;                                                        
   set _gtests_ point=i;
   set _counts_ point=i;
   ne(i) = Event;
   nc(i) = Censored;
   nt(i) = Total;
   e(i) = Estimate;                                                          
   s(i) = StdErr;
   z(i) = e(i)/s(i);                                                            
   p(i) = 1-probnorm(abs(z(i)));                                                
   p(i) = 2*p(i);                                                               
   lrchi(i) = ChiSq;
   plr(i) = ProbChisq;
   r(i) = exp( &&units&j* e(i));                                                
   u(i) = exp( &&units&j* (e(I)+z05*s(I)) );                                    
   l(i) = exp( &&units&j* (e(I)-z05*s(I)) );                                    
  end;                                                                          
  output;                                                                       
  stop;                                                                         
 %end;
  run;                                                                          
%end;                                                                           

%if %eval(&nest) gt 0 %then %do;
 data &estdata;
 %let upp = %eval(&nest*&ndep);
  keep e1-e&upp s1-s&upp z1-z&upp p1-p&upp
       r1-r&upp u1-u&upp l1-l&upp;
  array e(*) e1-e&upp;
  array s(*) s1-s&upp;
  array z(*) z1-z&upp;
  array p(*) p1-p&upp;
  array r(*) r1-r&upp;
  array u(*) u1-u&upp;
  array l(*) l1-l&upp;
  z05 =PROBIT(1-0.05/2);                                                        
  do i = 1 to totobs;
   set _estimates_ point=i nobs=totobs;
   e(i) = Estimate;
   s(i) = StdErr;
   z(i) = zValue;
   p(i) = Probz;
   r(i) = exp(e(i));                                                
   u(i) = exp((e(I)+z05*s(I)) );                                    
   l(i) = exp((e(I)-z05*s(I)) );                                    
  end;
  output;
  stop;
  run;
%end;

%if %eval(&ncon) gt 0 %then %do;
 data &condata ;
 %let upp = %eval(&ncon*&ndep);
 keep chisq1-chisq&upp p1-p&upp;
 array k(*) chisq1-chisq&upp;
 array p(*) p1-p&upp;
 do i = 1 to totobs;
  set _contrasts_ point=i nobs=totobs;
  k(i) = WaldChiSq;
  p(i) = ProbChiSq;
 end;
 output;
 stop;
 run;
%end;


%skipout:                                                                       
 proc delete data=_est_ _gtests_ _counts_ ;
run;

%mend ;                                                                         
