/**************************************************************************
   Updated March 2001 to use ODS for simpler programming and correct
   t and p values that were sometimes in error due to incorrect DF
   Works in Version 8 only
   Updated Jan 11, 2006 to add 95% CI for betas
**************************************************************************/
	       
%macro regp(data=_last_,dlist=,ilist=,blist=,factor=,out=&factor,
            cohort=Y,where=);
 
/* Overview:
     This macro runs PROC REG's and stores regression coefficients,
     standard errors, t-statistics, and p-values of covariates
     from the regression into one or more SAS datasets. 
 
     data   = data set to be read
     dlist  = dependent variable list
     ilist  = independent variable list (same for each dep. var)
     blist  = (optional) if given, the i'th variable is added as
              a covariate to the i'th analysis. Used mostly to
              adjust for the baseline value of the dependent
              variable.
     factor = independent variable(s) for which statistics are to be
              stored. 
     out    = output SAS dataset(s) in which the statistics
              are stored. There is one dataset for each variable in
	      factor with one observation in each dataset.
	      Statistics are named as:
 
               regression coefficients - b1-b?
               standard errors of coef - seb1-seb?
               t-statistics (coef/SE)  - t1-t?
               p-values                - p1-p?
	       lower 95% CIs           - l1-l?
	       upper 95% CIS           - u1-u?
 
               where ? is the number of dependent variables
               The default names for the datasets are &factor
     cohort  = if set to Y then a single PROC REG is run which excludes
               cases with any missing data from all regressions.
               If set to N then a PROC REG is run for each dependent
               variable.
     where   = where clause to subset observations
               (set where to "where sex = 1" to do analyses on men)
 
EXAMPLE:
%regp(data=temp,dlist=dbp3 sbp3,ilist=age sex, factor=age);

The datset age would contain the regression coefficients for age 
in variables b1 and b2.
*/
 
 
data _null_;
ods output ParameterEstimates 
           (match_all=_estimates_ persist=proc)= _estimates;
%local i j;
%let dlist = %upcase(&dlist) ;
%let factor = %upcase(&factor) ;
%let cohort = %upcase(&cohort) ;
 
%ntok(&factor,nfactor,vname=factor);
%ntok(&out ,nout ,vname=out);
%ntok(&dlist ,ndep);
%ntok(&ilist ,nind);
%ntok(&blist ,nbas,vname=bl);
 
 
%if &dlist eq %str() or %length(&ilist) = 0
    or %length(&factor)  eq 0  or &out eq %str() %then %do ;
 ERROR '*** SOME PARAMETERS NOT SPECIFIED - SEE MACRO CALL ***';
 ABORT;
 ENDSAS;
%end;
 
%if &blist ne %str() %then %do;
%if &ndep ne &nbas %then %do;
 ERROR '*** NUMBER OF DEPENDENT AND BASELINE VARIABLES NOT = ***';
 ABORT;
 ENDSAS;
%end;
%end;
 
%if &nout ne &nfactor %then %do ;
 ERROR '*** NUMBERS OF FACTORS AND OUTPUT DATASETS NOT =  ***';
 ABORT;
 ENDSAS;
%end;
 
 
%if &cohort eq Y %then %do;

 proc reg data=&data ;
 %if &blist eq %str() %then %do;
  model &dlist = &ilist/clb ;
 %end;

 %if &blist ne %str() %then %do;
  %do i = 1 %to &ndep;
   %let dep = %scan(&dlist,&i);
   model &dep = &ilist &&bl&i/clb;
  %end;
 %end;

 &where;
%end;
 
 
 %if &cohort ne Y %then %do ;

 %do i = 1 %to &ndep;
  %let dep = %scan(&dlist,&i);

  %if &blist eq %str() %then %do;
  proc reg data=&data ;
   model &dep = &ilist/clb ;
   &where;
  %end ;

  %if &blist ne %str() %then %do;
  proc reg data=&data ;
   model &dep = &ilist &&bl&i/clb;
   &where;
  %end ;

 %end;
 %end;

data _estimates_;
 set &_estimates_;
run;
 
%do j = 1 %to &nfactor ;
 data &&out&j;
  set _estimates_;
  if upcase(Variable) = "&&factor&j";
 data &&out&j;
  keep  p1-p&ndep b1-b&ndep seb1-seb&ndep t1-t&ndep l1-l&ndep u1-u&ndep;;
  array p(*) p1-p&ndep;
  array t(*) t1-t&ndep;
  array b(*) b1-b&ndep;
  array seb(*) seb1-seb&ndep;
  array l(*) l1-l&ndep;
  array u(*) u1-u&ndep;
 
  do i = 1 to totobs;
   set &&out&j point=i nobs=totobs;
    b(i) = Estimate;
    seb(i) = StdErr;
    t(i)   = tValue;
    p(i) = Probt ;
    l(i) = LowerCL;
    u(i) = UpperCL;
  end;
  output;
  stop;
  run;
%end;
ods output close;
proc delete data=_estimates_ ;
 
%mend ;
